@@ -1,8 +1,10 @@
 import warnings
 import re
+
 from collections import Counter
 
 from lxml import etree
+
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import relationship, backref, object_session
 from sqlalchemy.ext.associationproxy import association_proxy
@@ -15,6 +17,10 @@ from sqlalchemy.orm import sessionmaker
 
 from six import string_types as basestring
 
+from psims.utils import KeyToAttrProxy
+from .entity import Entity
+
+
 try:
     has_pyteomics = True
     from pyteomics.mass.mass import Composition, _make_isotope_string
@@ -124,7 +130,7 @@ def _formula_parser(formula, session):
                 count = int(count)
             else:
                 count = 1
-            if isotope is not None:
+            if isotope:
                 name = _make_isotope_string(element, int(isotope))
             else:
                 name = element
@@ -319,8 +325,8 @@ class Brick(Base, HasFullNameMixin):
         composition = CompositionType()
         for element_relation in self.elements:
             symbol = element_relation.element
-            isotope, element = re.search(r"(?P<isotope>\d*?)(?P<element>\S+)", symbol).groups()
-            if isotope != "":
+            isotope, element = re.search(r"(?P<isotope>\d+)?(?P<element>\S+)", symbol).groups()
+            if isotope:
                 isotope = int(isotope)
                 iso_str = _make_isotope_string(element, isotope)
             else:
@@ -355,13 +361,13 @@ class Fragment(Base):
         session = object_session(self)
         for fragment_composition_relation in self._fragment_composition:
             symbol = fragment_composition_relation.brick_string
-            isotope, element = re.search(r"(?P<isotope>\d*)?(?P<element>\S+)", symbol).groups()
+            isotope, element = re.search(r"(?P<isotope>\d+)?(?P<element>\S+)", symbol).groups()
             count = fragment_composition_relation.count
             if count is not None:
                 count = int(count)
             else:
                 count = 1
-            if isotope != "":
+            if isotope:
                 name = _make_isotope_string(element, isotope)
             else:
                 name = element
@@ -483,6 +489,7 @@ class Modification(Base, HasFullNameMixin):
     notes = relationship("MiscNotesModifications")
     specificities = relationship("Specificity")
     bricks = relationship(ModificationToBrick)
+    crossreferences = relationship("Crossreference")
     _fragments = relationship(Fragment)
 
     _alt_names = relationship(AlternativeName, backref=backref("modification"))
@@ -513,6 +520,10 @@ class Modification(Base, HasFullNameMixin):
                     inst.notes.append(model_note)
         return inst
 
+    @property
+    def name(self):
+        return self.ex_code_name
+
 
 class MiscNotesModifications(Base):
     __tablename__ = "MiscNotesModifications"
@@ -660,6 +671,14 @@ class Crossreference(Base):
         return inst
 
 
+class History(Base):
+    __tablename__ = "History"
+
+    id = Column(Integer, primary_key=True)
+    url = Column(Unicode(128))
+    version = Column(Unicode(128))
+
+
 def create(doc_path, output_path="sqlite://"):
     '''
     Parse the relational table-like XML file provided by http://www.unimod.org/downloads.html
@@ -678,6 +697,15 @@ def create(doc_path, output_path="sqlite://"):
                 for tag in tree.iterfind(".//" + model._tag_name):
                     session.add(model.from_tag(tag))
             session.commit()
+        root = tree.getroot()
+        version = "%s.%s" % (root.attrib['majorVersion'], root.attrib['minorVersion'])
+        if not isinstance(doc_path, basestring):
+            try:
+                doc_path = doc_path.name
+            except AttributeError:
+                doc_path = str(doc_path)
+        session.add(History(url=doc_path, version=version))
+        session.commit()
     return session
 
 
@@ -689,10 +717,13 @@ def session(path="sqlite:///unimod.db"):
 
 
 class Unimod(object):
-    def __init__(self, path=None):
+    name = "UNIMOD"
+    default_version = '1.0'
+
+    def __init__(self, path=None, unimod_xml_uri=_unimod_xml_download_url):
         if path is None:
             self.path = None
-            self.session = create(_unimod_xml_download_url)
+            self.session = create(unimod_xml_uri)
         else:
             self.path = path
             try:
@@ -701,13 +732,33 @@ class Unimod(object):
                     raise Exception()
             except Exception:
                 # Database may not yet exist at that location
-                self.session = create(_unimod_xml_download_url, path)
+                self.session = create(unimod_xml_uri, path)
                 self.session.query(Modification).first()
 
+    @property
+    def version(self):
+        try:
+            version = self.session.query(History.version).all()
+            if len(version) > 0:
+                version = max(version)
+            return version[0]
+        except Exception:
+            return self.default_version
+
     def get(self, identifier, strict=True):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=sa_exc.SAWarning)
-            if isinstance(identifier, int):
+            is_explicit_accession = isinstance(identifier, basestring) and identifier.startswith("UNIMOD")
+            try:
+                # At least one Modification has an empty string code_name or ex_code_name, causing
+                # this fuzzy finder function to happily respond to that record
+                if identifier == "":
+                    raise KeyError(identifier)
+            except (TypeError, ValueError):
+                pass
+            if isinstance(identifier, int) or is_explicit_accession:
+                if is_explicit_accession:
+                    identifier = int(identifier.replace("UNIMOD:", ''))
                 mod = self.session.query(Modification).get(identifier)
                 if mod is None:
                     raise KeyError(identifier)
@@ -738,11 +789,19 @@ class Unimod(object):
                             raise KeyError(identifier)
                         mod = alt_name.modification
                     return mod
+            else:
+                raise KeyError(identifier)
 
     by_title = by_name = get
 
     __getitem__ = get
 
+    def infer(self, mass, amino_acid, mass_error=1e-6):
+        candidates = self.session.query(Modification).filter(
+            Modification.monoisotopic_mass.between(mass - mass_error, mass + mass_error)
+        ).join(Modification.specificities).filter(Specificity.amino_acid == amino_acid).all()
+        return candidates
+
     @property
     def mods(self):
         return self.session.query(Modification).all()
@@ -753,3 +812,23 @@ class Unimod(object):
 
 def load(path=None):
     return Unimod(path)
+
+
+class UNIMODEntity(Entity):
+
+    def is_of_type(self, tp):
+        try:
+            if tp.startswith('UNIMOD'):
+                return True
+            return False
+        except AttributeError:
+            if isinstance(tp, UNIMODEntity):
+                return True
+
+    @classmethod
+    def converter(cls, modification, vocabulary):
+        data = dict(KeyToAttrProxy(modification))
+        data['id'] = 'UNIMOD:%s' % modification.id
+        data['name'] = modification.ex_code_name or modification.code_name or modification.full_name
+        data['_object'] = modification
+        return cls(vocabulary, **data)
