@@ -119,10 +119,10 @@ class MzIdentMLTranslater(TransformerBase):
         try:
             analysis_software_list = next(self.reader.iterfind("AnalysisSoftwareList"))
         except StopIteration:
-            analysis_software_list = []
+            analysis_software_list = {}
 
         analysis_software_list = map(
-            self.format_analysis_software, analysis_software_list.get("AnalysisSoftware"))
+            self.format_analysis_software, analysis_software_list.get("AnalysisSoftware", []))
 
         with self.writer.AnalysisSoftwareList(analysis_software_list):
             self.insert_software_record()
@@ -185,11 +185,11 @@ class MzIdentMLTranslater(TransformerBase):
         with writer.sequence_collection():
             reader.reset()
             i = 0
-            for db_seq in map(self.format_db_sequence, reader.iterfind("dBSequence")):
+            for db_seq in map(self.format_db_sequence, reader.iterfind("DBSequence")):
                 i += 1
                 db_seq.write(self.writer)
                 if i % K == 0:
-                    self.log("Copied %d dBSequences" % i)
+                    self.log("Copied %d DBSequences" % i)
                 if i > N:
                     break
 
@@ -215,7 +215,7 @@ class MzIdentMLTranslater(TransformerBase):
 
     def format_db_sequence(self, db_sequence):
         d = dict(db_sequence)
-        d['search_database_id'] = d.pop("searchDatabase_ref")
+        d['search_database_id'] = d.pop("searchDatabase_ref", None)
         d['sequence'] = d.pop("Seq", None)
         return self.writer.DBSequence.ensure(d)
 
@@ -363,13 +363,13 @@ class MzIdentMLTranslater(TransformerBase):
         d['parent_tolerance'] = self.format_tolerance(d.pop("ParentTolerance", {}), self.writer.ParentTolerance)
         d['fragment_tolerance'] = self.format_tolerance(d.pop("FragmentTolerance", {}), self.writer.FragmentTolerance)
         d['threshold'] = self._extract_params(d.pop("Threshold"))
-        d['analysis_software_id'] = d.pop("analysisSoftware_ref")
+        d['analysis_software_id'] = d.pop("analysisSoftware_ref", None)
         return self.writer.SpectrumIdentificationProtocol.ensure(d)
 
     def format_protein_detection_protocol(self, pdp):
         d = dict(pdp)
         d['threshold'] = self._extract_params(d.pop("Threshold"))
-        d['analysis_software_id'] = d.pop("analysisSoftware_ref")
+        d['analysis_software_id'] = d.pop("analysisSoftware_ref", None)
         d['params'] = d.pop("AnalysisParams", [])
         return self.writer.ProteinDetectionProtocol.ensure(d)
 
@@ -456,10 +456,14 @@ class MzIdentMLTranslater(TransformerBase):
                     for spectrum_id_result in spectrum_identification_list.pop("SpectrumIdentificationResult", []):
                         i += 1
                         result = self.format_spectrum_identification_result(spectrum_id_result)
+                        if result is None:
+                            continue
                         with result:
                             for item in map(
                                     self.format_spectrum_identification_item,
                                     spectrum_id_result['SpectrumIdentificationItem']):
+                                if item is None:
+                                    continue
                                 item.write(self.writer)
                         if i % K == 0:
                             self.log("Copied %d SpectrumIdentificationResults" % i)
@@ -475,14 +479,20 @@ class MzIdentMLTranslater(TransformerBase):
                     for pag in protein_detection_list.pop('ProteinAmbiguityGroup'):
                         i += 1
                         result = self.format_protein_ambiguity_group(pag)
+                        if result is None:
+                            continue
                         with result:
                             j = 0
                             for prot in pag['ProteinDetectionHypothesis']:
                                 result = self.format_protein_detection_hypothesis(prot)
+                                if result is None:
+                                    continue
                                 with result:
                                     k = 0
                                     for pept in prot['PeptideHypothesis']:
                                         result = self.format_peptide_detection_hypothesis(pept)
+                                        if result is None:
+                                            continue
                                         result.write(self.writer)
                                     k += 1
                                     if k > N:
